import { getModels } from '@/services/pluginsManager/map/models/getModels';
import { OpenMapArgs, openMap } from '@/services/pluginsManager/map/openMap';
import { triggerSearch } from '@/services/pluginsManager/map/triggerSearch';
import { MinervaConfiguration } from '@/services/pluginsManager/pluginsManager';
import { MapModel } from '@/types/models';
import { getDisease } from '@/services/pluginsManager/project/data/getDisease';
import { getName } from '@/services/pluginsManager/project/data/getName';
import { getOrganism } from '@/services/pluginsManager/project/data/getOrganism';
import { getProjectId } from '@/services/pluginsManager/project/data/getProjectId';
import { getVersion } from '@/services/pluginsManager/project/data/getVersion';

type Plugin = {
  pluginName: string;
  pluginVersion: string;
  pluginUrl: string;
};

type RegisterPlugin = ({ pluginName, pluginVersion, pluginUrl }: Plugin) => {
  element: HTMLDivElement;
};

type HashPlugin = {
  pluginUrl: string;
  pluginScript: string;
};

declare global {
  interface Window {
    minerva: {
      configuration?: MinervaConfiguration;
      plugins: {
        registerPlugin: RegisterPlugin;
      };
      data: {
        bioEntities: BioEntitiesMethods;
      };
      map: {
        data: {
          getModels: typeof getModels;
        };
        openMap: typeof openMap;
        triggerSearch: typeof triggerSearch;
      };
      project: {
        data: {
          getProjectId: typeof getProjectId;
          getName: typeof getName;
          getVersion: typeof getVersion;
          getDisease: typeof getDisease;
          getOrganism: typeof getOrganism;
        };
      };
    };
  }
}
