/* eslint-disable no-magic-numbers */
import { pluginFixture } from '@/models/fixtures/pluginFixture';
import { INITIAL_STORE_STATE_MOCK } from '@/redux/root/root.fixtures';
import { PluginsManager } from '@/services/pluginsManager';
import { getReduxStoreWithActionsListener } from '@/utils/testing/getReduxStoreActionsListener';
import { renderHook, waitFor } from '@testing-library/react';
import axios, { HttpStatusCode } from 'axios';
import MockAdapter from 'axios-mock-adapter';
import { act } from 'react-dom/test-utils';
import { useLoadPlugin } from './useLoadPlugin';

const mockedAxiosClient = new MockAdapter(axios);
jest.mock('../../../../../../services/pluginsManager/pluginsManager');

describe('useLoadPlugin', () => {
  afterEach(() => {
    jest.restoreAllMocks();
  });
  it('should unload plugin successfully', async () => {
    const { Wrapper, store } = getReduxStoreWithActionsListener({
      ...INITIAL_STORE_STATE_MOCK,
      plugins: {
        ...INITIAL_STORE_STATE_MOCK.plugins,
        activePlugins: {
          pluginsId: [pluginFixture.hash],
          data: {
            [pluginFixture.hash]: pluginFixture,
          },
        },
        list: INITIAL_STORE_STATE_MOCK.plugins.list,
      },
    });

    const {
      result: {
        current: { isPluginActive, isPluginLoading, togglePlugin },
      },
    } = renderHook(
      () => useLoadPlugin({ hash: pluginFixture.hash, pluginUrl: pluginFixture.urls[0] }),
      {
        wrapper: Wrapper,
      },
    );

    expect(isPluginActive).toBe(true);
    expect(isPluginLoading).toBe(false);

    act(() => {
      togglePlugin();
    });

    const actions = store.getActions();
    expect(actions[0]).toEqual({
      payload: { pluginId: pluginFixture.hash },
      type: 'plugins/removePlugin',
    });
  });
  it('should load plugin successfully', async () => {
    const hash = 'pluginHash';
    const pluginUrl = 'http://example.com/plugin.js';
    Math.max = jest.fn();
    const { Wrapper } = getReduxStoreWithActionsListener(INITIAL_STORE_STATE_MOCK);
    const pluginScript = `function init() {${Math.max(1, 2)}} init()`;

    mockedAxiosClient.onGet(pluginUrl).reply(HttpStatusCode.Ok, pluginScript);

    const {
      result: {
        current: { isPluginActive, isPluginLoading, togglePlugin },
      },
    } = renderHook(() => useLoadPlugin({ hash, pluginUrl }), {
      wrapper: Wrapper,
    });

    expect(isPluginActive).toBe(false);
    expect(isPluginLoading).toBe(false);

    togglePlugin();

    expect(Math.max).toHaveBeenCalledWith(1, 2);

    await waitFor(() => {
      expect(PluginsManager.setHashedPlugin).toHaveBeenCalledWith({
        pluginScript,
        pluginUrl,
      });
    });
  });
});
