/* eslint-disable no-magic-numbers */
import { compartmentPathwaysDetailsFixture } from '@/models/fixtures/compartmentPathways';
import { configurationFixture } from '@/models/fixtures/configurationFixture';
import { modelsFixture } from '@/models/fixtures/modelsFixture';
import { statisticsFixture } from '@/models/fixtures/statisticsFixture';
import { apiPath } from '@/redux/apiPath';
import { CONFIGURATION_INITIAL_STORE_MOCK } from '@/redux/configuration/configuration.mock';
import { INITIAL_STORE_STATE_MOCK } from '@/redux/root/root.fixtures';
import { AppDispatch, RootState } from '@/redux/store';
import { mockNetworkNewAPIResponse } from '@/utils/mockNetworkResponse';
import { getReduxStoreWithActionsListener } from '@/utils/testing/getReduxStoreActionsListener';
import { InitialStoreState } from '@/utils/testing/getReduxWrapperWithStore';
import { render, screen } from '@testing-library/react';
import { HttpStatusCode } from 'axios';
import { act } from 'react-dom/test-utils';
import { MockStoreEnhanced } from 'redux-mock-store';
import { ELEMENTS_COLUMNS } from '../ExportCompound/ExportCompound.constant';
import { Elements } from './Elements.component';

const mockedAxiosClient = mockNetworkNewAPIResponse();

const renderComponent = (
  initialStore?: InitialStoreState,
): { store: MockStoreEnhanced<Partial<RootState>, AppDispatch> } => {
  const { Wrapper, store } = getReduxStoreWithActionsListener(initialStore);
  return (
    render(
      <Wrapper>
        <Elements />
      </Wrapper>,
    ),
    {
      store,
    }
  );
};

describe('Elements - component', () => {
  it('should render all elements sections', () => {
    renderComponent({
      ...INITIAL_STORE_STATE_MOCK,
      configuration: {
        ...CONFIGURATION_INITIAL_STORE_MOCK,
        main: {
          ...CONFIGURATION_INITIAL_STORE_MOCK.main,
          data: {
            ...configurationFixture,
            miriamTypes: {
              compartment_label: {
                commonName: 'Compartment',
                homepage: '',
                registryIdentifier: '',
                uris: [''],
              },
            },
          },
        },
      },
      statistics: {
        data: {
          ...statisticsFixture,
          elementAnnotations: {
            compartment_label: 1,
            pathway: 0,
          },
        },
        loading: 'succeeded',
        error: {
          message: '',
          name: '',
        },
      },
      compartmentPathways: {
        data: compartmentPathwaysDetailsFixture,
        loading: 'succeeded',
        error: {
          message: '',
          name: '',
        },
      },
    });

    const annotations = screen.getByText('Select annotations');
    const includedCompartmentPathways = screen.getByText('Select included compartment / pathways');
    const excludedCompartmentPathways = screen.getByText('Select excluded compartment / pathways');
    const downloadButton = screen.getByText('Download');

    expect(annotations).toBeVisible();
    expect(includedCompartmentPathways).toBeVisible();
    expect(excludedCompartmentPathways).toBeVisible();
    expect(downloadButton).toBeVisible();
  });
  it('should handle download button click and dispatch proper data', async () => {
    mockedAxiosClient.onPost(apiPath.downloadElementsCsv()).reply(HttpStatusCode.Ok, 'test');
    const FIRST_COMPARMENT_PATHWAY_NAME = compartmentPathwaysDetailsFixture[0].name;
    const FIRST_COMPARMENT_PATHWAY_ID = compartmentPathwaysDetailsFixture[0].id;
    const SECOND_COMPARMENT_PATHWAY_NAME = compartmentPathwaysDetailsFixture[1].name;
    const SECOND_COMPARMENT_PATHWAY_ID = compartmentPathwaysDetailsFixture[1].id;
    const { store } = renderComponent({
      ...INITIAL_STORE_STATE_MOCK,
      configuration: {
        ...CONFIGURATION_INITIAL_STORE_MOCK,
        main: {
          ...CONFIGURATION_INITIAL_STORE_MOCK.main,
          data: {
            ...configurationFixture,
            miriamTypes: {
              compartment_label: {
                commonName: 'Compartment',
                homepage: '',
                registryIdentifier: '',
                uris: [''],
              },
            },
          },
        },
      },
      statistics: {
        data: {
          ...statisticsFixture,
          elementAnnotations: {
            compartment_label: 1,
            pathway: 0,
          },
        },
        loading: 'succeeded',
        error: {
          message: '',
          name: '',
        },
      },
      compartmentPathways: {
        data: compartmentPathwaysDetailsFixture,
        loading: 'succeeded',
        error: {
          message: '',
          name: '',
        },
      },
      models: {
        data: modelsFixture,
        loading: 'succeeded',
        error: {
          message: '',
          name: '',
        },
      },
    });

    const annotations = screen.getByText('Select annotations');

    await act(() => {
      annotations.click();
    });
    const annotationInput = screen.getByLabelText('Compartment');

    await act(() => {
      annotationInput.click();
    });

    expect(annotationInput).toBeChecked();

    const includedCompartmentPathways = screen.getByText('Select included compartment / pathways');

    await act(() => {
      includedCompartmentPathways.click();
    });
    const includedCompartmentPathwaysInput = screen.getAllByLabelText(
      FIRST_COMPARMENT_PATHWAY_NAME,
    )[0];

    await act(() => {
      includedCompartmentPathwaysInput.click();
    });

    expect(includedCompartmentPathwaysInput).toBeChecked();

    const excludedCompartmentPathways = screen.getByText('Select excluded compartment / pathways');

    await act(() => {
      excludedCompartmentPathways.click();
    });
    const excludedCompartmentPathwaysInput = screen.getAllByLabelText(
      SECOND_COMPARMENT_PATHWAY_NAME,
    )[1];

    await act(() => {
      excludedCompartmentPathwaysInput.click();
    });

    expect(excludedCompartmentPathwaysInput).toBeChecked();

    const downloadButton = screen.getByText('Download');

    await act(() => {
      downloadButton.click();
    });

    const actions = store.getActions();

    const firstAction = actions[0];
    expect(firstAction.meta.arg).toEqual({
      columns: ELEMENTS_COLUMNS,
      submaps: modelsFixture.map(item => item.idObject),
      annotations: ['compartment_label'],
      includedCompartmentIds: [FIRST_COMPARMENT_PATHWAY_ID],
      excludedCompartmentIds: [SECOND_COMPARMENT_PATHWAY_ID],
    });
  });
});
