import { currentLegendImagesSelector, legendSelector } from '@/redux/legend/legend.selectors';
import { StoreType } from '@/redux/store';
import {
  InitialStoreState,
  getReduxWrapperWithStore,
} from '@/utils/testing/getReduxWrapperWithStore';
import { render, screen, within } from '@testing-library/react';
import { Legend } from './Legend.component';
import { LEGEND_ROLE } from './Legend.constants';

jest.mock('../../../redux/legend/legend.selectors', () => ({
  legendSelector: jest.fn(),
  currentLegendImagesSelector: jest.fn(),
}));

const legendSelectorMock = legendSelector as unknown as jest.Mock;
const currentLegendImagesSelectorMock = currentLegendImagesSelector as unknown as jest.Mock;

const renderComponent = (initialStore?: InitialStoreState): { store: StoreType } => {
  const { Wrapper, store } = getReduxWrapperWithStore(initialStore);
  return (
    render(
      <Wrapper>
        <Legend />
      </Wrapper>,
    ),
    {
      store,
    }
  );
};

describe('Legend - component', () => {
  beforeAll(() => {
    currentLegendImagesSelectorMock.mockImplementation(() => []);
  });

  describe('when is closed', () => {
    beforeEach(() => {
      legendSelectorMock.mockImplementation(() => ({
        isOpen: false,
      }));
      renderComponent();
    });

    it('should render the component without translation', () => {
      expect(screen.getByRole(LEGEND_ROLE)).not.toHaveClass('translate-y-0');
    });
  });

  describe('when is open', () => {
    beforeEach(() => {
      legendSelectorMock.mockImplementation(() => ({
        isOpen: true,
      }));
      renderComponent();
    });

    it('should render the component with translation', () => {
      expect(screen.getByRole(LEGEND_ROLE)).toHaveClass('translate-y-0');
    });

    it('should render legend header', async () => {
      const legendContainer = screen.getByRole(LEGEND_ROLE);
      const legendHeader = await within(legendContainer).getByTestId('legend-header');
      expect(legendHeader).toBeInTheDocument();
    });

    it('should render legend images', async () => {
      const legendContainer = screen.getByRole(LEGEND_ROLE);
      const legendImages = await within(legendContainer).getByTestId('legend-images');
      expect(legendImages).toBeInTheDocument();
    });
  });
});
