import { FIRST_ARRAY_ELEMENT } from '@/constants/common';
import {
  BACKGROUNDS_MOCK,
  BACKGROUND_INITIAL_STATE_MOCK,
} from '@/redux/backgrounds/background.mock';
import { initialMapStateFixture } from '@/redux/map/map.fixtures';
import { AppDispatch, RootState, StoreType } from '@/redux/store';
import { getReduxStoreWithActionsListener } from '@/utils/testing/getReduxStoreActionsListener';
import {
  InitialStoreState,
  getReduxWrapperWithStore,
} from '@/utils/testing/getReduxWrapperWithStore';
import { render, screen } from '@testing-library/react';
import { MockStoreEnhanced } from 'redux-mock-store';
import { MapAdditionalOptions } from './MapAdditionalOptions.component';

const renderComponent = (initialStoreState: InitialStoreState = {}): { store: StoreType } => {
  const { Wrapper, store } = getReduxWrapperWithStore(initialStoreState);

  return (
    render(
      <Wrapper>
        <MapAdditionalOptions />
      </Wrapper>,
    ),
    {
      store,
    }
  );
};

const renderComponentWithActionListener = (
  initialStoreState: InitialStoreState = {},
): { store: MockStoreEnhanced<Partial<RootState>, AppDispatch> } => {
  const { Wrapper, store } = getReduxStoreWithActionsListener(initialStoreState);

  return (
    render(
      <Wrapper>
        <MapAdditionalOptions />
      </Wrapper>,
    ),
    {
      store,
    }
  );
};

describe('MapAdditionalOptions - component', () => {
  it('should display background selector', () => {
    renderComponent();
    expect(screen.getByTestId('background-selector')).toBeInTheDocument();
  });

  it('should render browse overview images button', () => {
    renderComponent();
    expect(screen.getByText('Browse overview images')).toBeInTheDocument();
  });

  it('should open overview image modal on button click', () => {
    const { store } = renderComponentWithActionListener({
      map: initialMapStateFixture,
      backgrounds: { ...BACKGROUND_INITIAL_STATE_MOCK, data: BACKGROUNDS_MOCK },
    });

    const overviewImageButton = screen.getByText('Browse overview images');
    overviewImageButton.click();

    const actions = store.getActions();
    expect(actions[FIRST_ARRAY_ELEMENT]).toStrictEqual({
      payload: 0,
      type: 'modal/openOverviewImagesModalById',
    });
  });
});
