/* eslint-disable no-magic-numbers */
import { BACKGROUND_INITIAL_STATE_MOCK } from '@/redux/backgrounds/background.mock';
import { MAP_DATA_INITIAL_STATE, OPENED_MAPS_INITIAL_STATE } from '@/redux/map/map.constants';
import { initialMapStateFixture } from '@/redux/map/map.fixtures';
import { getReduxWrapperWithStore } from '@/utils/testing/getReduxWrapperWithStore';
import { renderHook, waitFor } from '@testing-library/react';
import { Map } from 'ol';
import BaseLayer from 'ol/layer/Base';
import TileLayer from 'ol/layer/Tile';
import VectorLayer from 'ol/layer/Vector';
import React from 'react';
import { useOlMap } from '../useOlMap';
import { useOlMapLayers } from './useOlMapLayers';

const useRefValue = {
  current: null,
};

Object.defineProperty(useRefValue, 'current', {
  get: jest.fn(() => ({
    innerHTML: '',
    appendChild: jest.fn(),
    addEventListener: jest.fn(),
    getRootNode: jest.fn(),
  })),
  set: jest.fn(() => ({
    innerHTML: '',
    appendChild: jest.fn(),
    addEventListener: jest.fn(),
    getRootNode: jest.fn(),
  })),
});

jest.spyOn(React, 'useRef').mockReturnValue(useRefValue);

describe('useOlMapLayers - util', () => {
  it('should modify layers of the map instance on init', async () => {
    const { Wrapper } = getReduxWrapperWithStore({
      map: initialMapStateFixture,
      backgrounds: BACKGROUND_INITIAL_STATE_MOCK,
    });

    const dummyElement = document.createElement('div');
    const mapInstance = new Map({ target: dummyElement });
    const setLayersSpy = jest.spyOn(mapInstance, 'setLayers');
    const CALLED_ONCE = 1;

    renderHook(() => useOlMapLayers({ mapInstance }), {
      wrapper: Wrapper,
    });

    await waitFor(() => expect(setLayersSpy).toBeCalledTimes(CALLED_ONCE));
  });

  const getRenderedHookResults = (): BaseLayer[] => {
    const { Wrapper } = getReduxWrapperWithStore({
      map: {
        data: {
          ...MAP_DATA_INITIAL_STATE,
          size: {
            width: 256,
            height: 256,
            tileSize: 256,
            minZoom: 1,
            maxZoom: 1,
          },
          position: {
            initial: {
              x: 256,
              y: 256,
            },
            last: {
              x: 256,
              y: 256,
            },
          },
        },
        loading: 'idle',
        error: {
          name: '',
          message: '',
        },
        openedMaps: OPENED_MAPS_INITIAL_STATE,
      },
    });
    const dummyElement = document.createElement('div');
    const { result: hohResult } = renderHook(() => useOlMap({ target: dummyElement }), {
      wrapper: Wrapper,
    });

    const { result } = renderHook(
      () => useOlMapLayers({ mapInstance: hohResult.current.mapInstance }),
      {
        wrapper: Wrapper,
      },
    );

    return result.current;
  };

  it('should return valid TileLayer instance [1]', () => {
    const result = getRenderedHookResults();

    expect(result[0]).toBeInstanceOf(TileLayer);
    expect(result[0].getSourceState()).toBe('ready');
  });

  it('should return valid VectorLayer instance [2]', () => {
    const result = getRenderedHookResults();

    expect(result[1]).toBeInstanceOf(VectorLayer);
    expect(result[1].getSourceState()).toBe('ready');
  });

  it('should return valid VectorLayer instance [3]', () => {
    const result = getRenderedHookResults();

    expect(result[2]).toBeInstanceOf(VectorLayer);
    expect(result[2].getSourceState()).toBe('ready');
  });
});
