import { SIZE_OF_EMPTY_ARRAY } from '@/constants/common';
import { rootSelector } from '@/redux/root/root.selectors';
import { MultiSearchData } from '@/types/fetchDataState';
import { BioEntity, BioEntityContent, MapModel } from '@/types/models';
import { createSelector } from '@reduxjs/toolkit';
import { searchedChemicalsBioEntitesOfCurrentMapSelector } from '../chemicals/chemicals.selectors';
import { currentSelectedBioEntityIdSelector } from '../contextMenu/contextMenu.selector';
import {
  currentSearchedBioEntityId,
  currentSelectedSearchElement,
} from '../drawer/drawer.selectors';
import { searchedDrugsBioEntitesOfCurrentMapSelector } from '../drugs/drugs.selectors';
import { currentModelIdSelector, modelsDataSelector } from '../models/models.selectors';

export const bioEntitySelector = createSelector(rootSelector, state => state.bioEntity);

export const bioEntityDataSelector = createSelector(bioEntitySelector, bioEntity => bioEntity.data);

export const bioEntityDataListSelector = createSelector(bioEntityDataSelector, bioEntityData =>
  bioEntityData.map(b => b.data || []).flat(),
);

export const bioEntitiesForSelectedSearchElement = createSelector(
  bioEntitySelector,
  currentSelectedSearchElement,
  (bioEntitiesState, currentSearchElement): MultiSearchData<BioEntityContent[]> | undefined =>
    bioEntitiesState.data.find(
      ({ searchQueryElement }) => searchQueryElement === currentSearchElement,
    ),
);

export const searchedFromMapBioEntityElement = createSelector(
  bioEntitiesForSelectedSearchElement,
  currentSearchedBioEntityId,
  (bioEntitiesState, currentBioEntityId): BioEntity | undefined =>
    bioEntitiesState &&
    bioEntitiesState.data?.find(({ bioEntity }) => bioEntity.id === currentBioEntityId)?.bioEntity,
);

export const searchedBioEntityElementForContextMapSelector = createSelector(
  bioEntitySelector,
  currentSelectedBioEntityIdSelector,
  (bioEntitiesState, currentBioEntityId): BioEntity | undefined => {
    return bioEntitiesState.data
      .find(({ searchQueryElement }) => searchQueryElement === currentBioEntityId.toString())
      ?.data?.find(({ bioEntity }) => bioEntity.id === currentBioEntityId)?.bioEntity;
  },
);

export const searchedBioEntityElementUniProtIdSelector = createSelector(
  searchedBioEntityElementForContextMapSelector,
  (bioEntitiesState): string | undefined => {
    return bioEntitiesState?.references.find(({ type }) => type === 'UNIPROT')?.resource;
  },
);

export const searchedFromMapBioEntityElementRelatedSubmapSelector = createSelector(
  searchedFromMapBioEntityElement,
  modelsDataSelector,
  (bioEntity, models): MapModel | undefined =>
    models.find(({ idObject }) => idObject === bioEntity?.submodel?.mapId),
);

export const loadingBioEntityStatusSelector = createSelector(
  bioEntitiesForSelectedSearchElement,
  state => state?.loading,
);

export const searchedBioEntitesSelectorOfCurrentMap = createSelector(
  bioEntitySelector,
  currentSelectedSearchElement,
  currentModelIdSelector,
  (bioEntities, currentSearchElement, currentModelId): BioEntity[] => {
    if (!bioEntities) {
      return [];
    }

    return (bioEntities?.data || [])
      .filter(({ searchQueryElement }) =>
        currentSearchElement ? searchQueryElement === currentSearchElement : true,
      )
      .map(({ data }) => data || [])
      .flat()
      .filter(({ bioEntity }) => bioEntity.model === currentModelId)
      .map(({ bioEntity }) => bioEntity);
  },
);

export const numberOfBioEntitiesSelector = createSelector(
  bioEntitiesForSelectedSearchElement,
  state => (state?.data ? state.data.length : SIZE_OF_EMPTY_ARRAY),
);

export const bioEntitiesPerModelSelector = createSelector(
  bioEntitiesForSelectedSearchElement,
  modelsDataSelector,
  (bioEntities, models) => {
    const bioEntitiesPerModelPerSearchElement = (models || []).map(model => {
      const bioEntitiesInGivenModel = (bioEntities?.data || []).filter(
        entity => model.idObject === entity.bioEntity.model,
      );

      return {
        modelName: model.name,
        modelId: model.idObject,
        numberOfEntities: bioEntitiesInGivenModel.length,
        bioEntities: bioEntitiesInGivenModel,
      };
    });

    return bioEntitiesPerModelPerSearchElement.filter(
      model => model.numberOfEntities !== SIZE_OF_EMPTY_ARRAY,
    );
  },
);

export const allVisibleBioEntitiesSelector = createSelector(
  searchedBioEntitesSelectorOfCurrentMap,
  searchedChemicalsBioEntitesOfCurrentMapSelector,
  searchedDrugsBioEntitesOfCurrentMapSelector,
  (content, chemicals, drugs): BioEntity[] => {
    return [content, chemicals, drugs].flat();
  },
);
