import { ReferenceFiltered } from '@/types/reference';
import { getReferencesGroupedBySource } from './getGroupedReferences';

describe('getGroupedReferences - util', () => {
  const singleReference = {
    link: 'https://www.ncbi.nlm.nih.gov/pubmed/24448649',
    article: {
      title:
        'The nutrient-responsive transcription factor TFE3 promotes autophagy, lysosomal biogenesis, and clearance of cellular debris.',
      authors: [
        'Martina JA',
        ' Diab HI',
        ' Lishu L',
        ' Jeong-A L',
        ' Patange S',
        ' Raben N',
        ' Puertollano R.',
      ],
      journal: 'Science signaling',
      year: 2014,
      link: 'https://www.ncbi.nlm.nih.gov/pubmed/24448649',
      pubmedId: '24448649',
      citationCount: 321,
    },
    type: 'PUBMED',
    resource: '24448649',
    id: 154973,
    annotatorClassName: '',
  };

  const cases = [
    [[], []],
    [
      [singleReference],
      [
        {
          source: '',
          references: [singleReference],
        },
      ],
    ],
    [
      [
        {
          ...singleReference,
          annotatorClassName: 'source1',
        },
        {
          ...singleReference,
          annotatorClassName: 'source1',
        },
        {
          ...singleReference,
          annotatorClassName: 'source2',
        },
      ],
      [
        {
          source: 'source1',
          references: [
            {
              ...singleReference,
              annotatorClassName: 'source1',
            },
            {
              ...singleReference,
              annotatorClassName: 'source1',
            },
          ],
        },
        {
          source: 'source2',
          references: [
            {
              ...singleReference,
              annotatorClassName: 'source2',
            },
          ],
        },
      ],
    ],
  ];

  it.each(cases)('should return correct grouped references', (references, referencesGrouped) =>
    expect(getReferencesGroupedBySource(references as ReferenceFiltered[])).toMatchObject(
      referencesGrouped,
    ),
  );
});
