import { MapInstance } from '@/types/map';
import { Dispatch, SetStateAction, createContext, useContext, useMemo, useState } from 'react';

export interface MapInstanceContext {
  mapInstance: MapInstance;
  setMapInstance: Dispatch<SetStateAction<MapInstance>>;
}

export const MapInstanceContext = createContext<MapInstanceContext>({
  mapInstance: undefined,
  setMapInstance: () => {},
});

export const useMapInstance = (): MapInstanceContext => useContext(MapInstanceContext);

export interface MapInstanceProviderProps {
  children: React.ReactNode;
  initialValue?: MapInstanceContext;
}

export const MapInstanceProvider = ({
  children,
  initialValue,
}: MapInstanceProviderProps): JSX.Element => {
  const [mapInstance, setMapInstance] = useState<MapInstance>(initialValue?.mapInstance);

  const mapInstanceContextValue = useMemo(
    () => ({
      mapInstance,
      setMapInstance,
    }),
    [mapInstance],
  );

  return (
    <MapInstanceContext.Provider value={mapInstanceContextValue}>
      {children}
    </MapInstanceContext.Provider>
  );
};
